function [E,A,conf_true] = gen_SBM_with_cliques( N,c,q,p,seed )
% gen_SBM_with_cliques generates a Stochastic Block Model network given N(system size), c(average degree)
% q (number of groups), p (epsilon=c_out/c_in) and seed(random number
% seed), then adds 10 cliques (each of which has size 10) to the network.
% E is the edges list, A is the adjacency matrix and conf_true is the
% planted partition.
    fprintf('Generating SBM network with cliques, with %g nodes, %g groups, average degree %g, epsilon=%g\n',N,q,c,p) 
    assert(mod(N,q)==0);
    
    %% Generating SBM network
    conf_true=[];
    for a=1:q
        conf_true = [conf_true ; ones(round(N/q),1)*a]; % the planted partition
    end

    cin=q*c/(1+(q-1)*p);
    cout=cin*p;
    
    if(p>999)
        cin=0;
        cout=q*c/(q-1);
    end
    mmin=round(1/q*(N/q-1)/2*cin); % number of edges in the same group
    mout=round(1/q*(N/q)*cout); % number of edges in different groups
    while true; % do until good
%       reset(stream,seed);
        rng('default');
        rng(seed);
        E=[];
        for a=1:q
            for b=a:q
                begina=round(N/q*(a-1)+1);
                enda=round(N/q*a);
                beginb=round(N/q*(b-1)+1);
                endb=round(N/q*b);
                if b==a % group aa
                    m=mmin;
                else % group ab
                    m=mout;
                end
                E1a=randi([begina enda],[round(1.2*m) 1]);
                E1b=randi([beginb endb],[round(1.2*m) 1]);
                E1=[E1a E1b];
                while (1)
                    E1=E1(find(E1(:,1)~=E1(:,2)),:);
                    E1T=E1';
                    [tmp,idx]=unique([min(E1T,[],1);max(E1T,[],1)]','rows');
                    E1=E1(idx,:);
                    if(length(E1)>=mmin)
                        break
                    else
                        E1a=randi([begina enda],[round(0.2*m) 1]);
                        E1b=randi([beginb endb],[round(0.2*m) 1]);
                        E1=[E1;E1a E1b];
                    end
                end
                
                %E1=E1(1:m,:);
                idx=randperm(length(E1));
                E1=E1(idx(1:m),:);
                E=[E;E1];
            end
        end
        Nx=max(max(E));
        if(Nx == N) 
            break;
        end
        seed=seed+1000;
    end
    M=length(E);
    A = sparse(E(:,1),E(:,2),ones(M,1),N,N,M);
    A = A+A';
    E=A2E(A);  % Convert from sparse adjacency matrix to edge list
    
    %% randomlize the labels of the nodes
    idx=randperm(N);  
    E=idx(E);
    x=conf_true;
    x(idx)=conf_true;
    conf_true=x;
    A=E2A(E); % Convert from edge list to adjacency matrix

    
    %% Adding cliques
    fprintf('Adding 10 cliques...');
    n=length(A);
    size_clique=10;  % size of cliques
    num_clique=10;   % number of cliques
    prob=1;          % prob=1 => cliques prob < 1 => densely connected graphs instead of cliques
    bin=n/num_clique;
    for x=1:num_clique;
        for i=((x-1)*bin+1):((x-1)*bin+size_clique);
            for j=(x-1)*bin+1:(x-1)*bin+size_clique;
                if(i ~= j)
                    if(rand(1,1)<prob)
                        A(i,j)=1;
                        A(j,i)=1;
                    end
                end
            end
        end
    end
    E=A2E(A);
    fprintf(' done!\n')
    return
end

